package Chainmail::Address;
use Chainmail::Report;
use Chainmail::Config;

sub new 
  {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    my $self  = {};

    $self->{LOCAL_PART} = shift;
    $self->{DOMAIN} = shift;

    $self->{INFO} = shift;

    bless ($self, $class);
    return $self;
  }


sub get_address
  {
    my $self = shift;
    my $part;
    my $domain = undef;

    my $local_part = $self->get_local_part();

    if (!defined $local_part)
      {
	error( "apparently empty address" );
	return undef;
      }

    foreach $part (@{$self->{DOMAIN}})
      {
	$domain .= "$part.";
      }
    
    
    if ( !defined $domain )
      {
	$domain = Chainmail::Config::generate_domain( $local_part, $self->{INFO} );
      }
    else
      {
	chop $domain;
      }
#    debug( "constructed address $local_part\@$domain");

    return "$local_part\@$domain";
  }

sub get_local_part
  {
    my $self = shift;
    local( $_ );

    if ( (!defined @{$self->{LOCAL_PART}}) || !defined( @{$self->{LOCAL_PART}}[0]))
      {
	debug( "no local part to address (get_local_part)" );
	return undef;
      }

    foreach my $part (@{$self->{LOCAL_PART}})
      {
	if( $part =~ /^[A-Za-z_]([A-Za-z\-0-9_]*[A-Za-z0-9_])?$/ )
	  {
	    $_ .= "$part.";
	  }
	else
	  {
	    $_ .= "\"$part\".";
	  }
      }
    chop;

    return $_;

  }

sub get_info
  {
    my $self = shift;

    return undef
      if !defined $self->{INFO};


    $self->{INFO} = $1
      while $self->{INFO} =~/\"((.)*?)\"$/s;

    # Escape all unquoted quotes
    $self->{INFO} =~ s/(?<!\\)((?:\\\\)*)\"/$1\\\"/g;
    


    return $self->{INFO} ;
  }

sub guess_name
  {
    my $self = shift;

    my $name = $self->get_info() ;

    enter( "guessing name" );
    
    (leave(), return $name) if defined $name 
	;

    if ( (!defined @{$self->{LOCAL_PART}}) || !defined( @{$self->{LOCAL_PART}}[0]))
      {
	debug( "could not guess name because address is empty" );
	leave();
	return undef;
      }

    foreach my $part (@{$self->{LOCAL_PART}})
      {
	$name .= "$part ";
      }

    (debug( "found address has insinuously empty local part"), leave(), return undef)
      if length($name) == 0;

    chop $name;

    leave();
    return $name;
}    
    

sub get_text
  {
    my $self = shift;
    my $name = ($self->get_info() or undef);
#     $self->guess_name or (error("could not find any reasonable name"),return undef);
    $name = "\"$name\" "
      if defined $name;

    return ( $name or "" ) . "<" . 
      $self->get_address() . ">";
  }

sub transform
  {
    my $self = shift;

    if (!defined( $self->{LOCAL_PART}) || !defined( $self->{LOCAL_PART}[0] ))
      {
	debug( "attempt to transform an empty address!" );
	return undef;
      }

    else
      {
	return Chainmail::Config::transform_address( \$self );
      }
  }

1;
	
		  
